#!/bin/sh

# Include library scripts.
if [ -f "/opt/drweb/scripts/include/lib.sh" ]; then
    . "/opt/drweb/scripts/include/lib.sh"
elif [ -f "/usr/local/drweb/scripts/include/lib.sh" ]; then
    . "/usr/local/drweb/scripts/include/lib.sh"
else
    printf '\nScript library not found.\n\n'
    exit 1
fi

checkRoot

# Configuration files to modify.
MONITOR_CONF=$CONFDIR/monitor.conf
DRWEB32_INI=$CONFDIR/drweb32.ini
NSS_CONF=$CONFDIR/drweb-nss.conf
DAEMON_MMC=$CONFDIR/monitor/daemon.mmc

SCRIPT_NAME=`basename $0`

conf_failed() {
    echo $1
    echo "Configuration failed!"
    exit 1
}

ask_protected_nss() {
    # Check is there's anything in the $NSS_PATH at all.
    ls -d $NSS_PATH/*/ >/dev/null 2>&1
    [ "$?" = "0" ] || return

    #
    # Ask for protected NSS volumes.
    #

    nss_dirs=`ls -d $NSS_PATH/*/`
    for dir in $nss_dirs; do
        dir=`basename $dir`
        NSS_DIRS="$NSS_DIRS $dir"
    done
    N_NSS_DIRS=`echo $NSS_DIRS | wc -w`

    protected_volumes=`IniKeyValues $NSS_CONF_NEW NSS ProtectedVolumes`

    # Set defaults.
    n=1
    for dir in $NSS_DIRS; do
        if StringInList "$protected_volumes" "$dir"; then
            eval "a_$n=1"
        else
            eval "a_$n="
        fi
        n=`echo "$n 1 + p" | dc`
    done

    while true; do
        clear
        echo "Select the protected volumes in $NSS_PATH."
        echo
        n=1
        for dir in $NSS_DIRS; do
            printf "%3d) " $n
            r=`eval "echo \\$a_$n"`
            if [ -n "$r" ]; then
                echo -n "[X]"
            else
                echo -n "[ ]"
            fi
            echo " $dir"
            n=`echo "$n 1 + p" | dc`
        done
        echo
        echo "Enter directory number to toggle selection."
        echo "Enter A or All to select all directories."
        echo "Enter N or None to deselect all directories."
        echo "Enter 0, Q or Quit when done. "
        echo "All values are case insensitive."
        echo -n "Select: "
        read inp
        case "$inp" in
	    0 | [qQ] | [qQ][uU][iI][tT])
                break
                ;;
            [aA][lL][lL] | [aA])
                for n in `seq $N_NSS_DIRS`; do
                    eval "a_$n=1"
                done
                continue
                ;;
            [nN][oO][nN][eE] | [nN])
                for n in `seq $N_NSS_DIRS`; do
                    eval "a_$n="
                done
                continue
                ;;
        esac

        r=`eval "echo \\$a_$inp"`
        if [ -z "$r" ]; then
            eval "a_$inp=1"
        else
            eval "a_$inp="
        fi
    done
    echo

    # Put list of protected NSS volumes into $NSS_CONF_NEW.
    PROTECTED_VOLUMES="ProtectedVolumes ="
    n=1
    for dir in $NSS_DIRS; do
        r=`eval "echo \\$a_$n"`
        if [ -n "$r" ]; then
            PROTECTED_VOLUMES="$PROTECTED_VOLUMES $dir"
        fi
        n=`echo "$n 1 + p" | dc`
    done
    if IniHaveKey $NSS_CONF_NEW NSS ProtectedVolumes ; then
        IniSectProc $NSS_CONF_NEW NSS "s|^\\s*ProtectedVolumes\\s*=.*|$PROTECTED_VOLUMES|;"
    else
        IniSectProc $NSS_CONF_NEW NSS "if (!\$added) { \$_=\"# Added by Dr.Web's $SCRIPT_NAME\n$PROTECTED_VOLUMES\n\n\".\$_; \$added=1 }"
    fi
}

# Check if configuration files exist.
[ -f $MONITOR_CONF ] || conf_failed "$MONITOR_CONF is absent."
[ -f $DRWEB32_INI ] || conf_failed "$DRWEB32_INI is absent."
[ -f $NSS_CONF ] || conf_failed "$NSS_CONF is absent."
[ -f $DAEMON_MMC ] || conf_failed "$DAEMON_MMC is absent."

# Make temporary copies of configs to edit.
MONITOR_CONF_NEW=`MakeTmpFile`
cp -f $MONITOR_CONF $MONITOR_CONF_NEW
DRWEB32_INI_NEW=`MakeTmpFile`
cp -f $DRWEB32_INI $DRWEB32_INI_NEW
NSS_CONF_NEW=`MakeTmpFile`
cp -f $NSS_CONF $NSS_CONF_NEW
DAEMON_MMC_NEW=`MakeTmpFile`
cp -f $DAEMON_MMC $DAEMON_MMC_NEW

# Set "User" to "root" in $MONITOR_CONF_NEW.
echo
echo "Setting 'User' to 'root' in [Monitor] of $MONITOR_CONF ."
if [ -z "`IniKeyValues $MONITOR_CONF_NEW Monitor User`" ]; then
    IniSectProc $MONITOR_CONF_NEW Monitor "if (!\$added) { \$_=\"# Added by Dr.Web's $SCRIPT_NAME\nUser = root\n\n\".\$_; \$added=1 }"
else
    IniSectProc $MONITOR_CONF_NEW Monitor "s#^(\\s*User\\s*=\\s*).*#\$1root#;"
fi

# Add "NSS" to "RunAppList" in $MONITOR_CONF_NEW.
run_app_list=`IniKeyValues $MONITOR_CONF_NEW Monitor RunAppList`
echo "Adding 'NSS' to 'RunAppList' in [Monitor] of $MONITOR_CONF ."
if [ -z "$run_app_list" ]; then
    IniSectProc $MONITOR_CONF_NEW Monitor "if (!\$added) { \$_=\"# Added by Dr.Web's $SCRIPT_NAME\nRunAppList = NSS\n\n\".\$_; \$added=1 }"
else
    echo $run_app_list | grep -w NSS >/dev/null
    if [ "$?" != "0" ]; then
        to_add=NSS
        [ -n "$run_app_list" ] && to_add=", $to_add"
        IniSectProc $MONITOR_CONF_NEW Monitor "s#^(\\s*RunAppList\\s*=.*)#\$1$to_add#;"
    fi
fi

AskUpdateConfig $MONITOR_CONF $MONITOR_CONF_NEW dont_ask

# Set "User" to "root" in $DRWEB32_INI_NEW.
echo
echo "Setting 'User' to 'root' in [Daemon] of $DRWEB32_INI ."
if [ -z "`IniKeyValues $DRWEB32_INI_NEW Daemon User`" ]; then
    IniSectProc $DRWEB32_INI_NEW Daemon "if (!\$added) { \$_=\"# Added by Dr.Web's $SCRIPT_NAME\nUser = root\n\n\".\$_; \$added=1 }"
else
    IniSectProc $DRWEB32_INI_NEW Daemon "s#^(\\s*User\\s*=\\s*).*#\$1root#;"
fi

AskUpdateConfig $DRWEB32_INI $DRWEB32_INI_NEW dont_ask

# Ask for "NSSVolumesMountDir" and put it into $NSS_CONF_NEW.
#
nss_mount_dir=`IniKeyValues $NSS_CONF_NEW NSS NSSVolumesMountDir`
[ -n "$nss_mount_dir" ] && NSS_PATH_DEFAULT=$nss_mount_dir
[ -z "$nss_mount_dir" ] && NSS_PATH_DEFAULT=/media/nss

while [ -z "$NSS_PATH" ]; do
    echo
    echo -n "Enter the path to your NSS share [$NSS_PATH_DEFAULT]: "
    read NSS_PATH
    [ -n "$NSS_PATH" ] || NSS_PATH=$NSS_PATH_DEFAULT
    if [ ! -d "$NSS_PATH" ]; then
        echo "No such directory."
        NSS_PATH=
    fi
done
NSS_PATH=`echo $NSS_PATH | sed 's#/$##'`

echo
echo "Setting 'NSSVolumesMountDir' to '$NSS_PATH' in [NSS] of $NSS_CONF ."
if [ -z "$nss_mount_dir" ]; then
    IniSectProc $NSS_CONF_NEW NSS "if (!\$added) { \$_=\"# Added by Dr.Web's $SCRIPT_NAME\nNSSVolumesMountDir = $NSS_PATH\n\n\".\$_; \$added=1 }"
else
    IniSectProc $NSS_CONF_NEW NSS "s#^(\\s*NSSVolumesMountDir\\s*=\\s*).*#\$1$NSS_PATH#;"
fi

echo "Setting up 'ProtectedVolumes' in [NSS] of $NSS_CONF ."
echo
echo 'Do you want to select volumes to protect?'
if shownoyes '(answer "no" to protect all of them)'; then
    # Ask for protected NSS volumes and put the list into $NSS_CONF_NEW.
    ask_protected_nss
else
    # Clear ProtectedVolumes list (it means everything is protected).
    if IniHaveKey $NSS_CONF_NEW NSS ProtectedVolumes ; then
        IniSectProc $NSS_CONF_NEW NSS "s|^(\\s*ProtectedVolumes\\s*=).*|\$1|;"
    else
        IniSectProc $NSS_CONF_NEW NSS "if (!\$added) { \$_=\"# Added by Dr.Web's $SCRIPT_NAME\nProtectedVolumes =\n\n\".\$_; \$added=1 }"
    fi
fi

# Ask for drwebd address and put it into $NSS_CONF_NEW.
#
drwebd_address=`IniKeyValues $NSS_CONF_NEW DaemonCommunication Address`
[ -n "$drwebd_address" ] && DRWEBD_ADDRESS_DEFAULT=$drwebd_address
[ -z "$drwebd_address" ] && DRWEBD_ADDRESS_DEFAULT=pid:/var/drweb/run/drwebd.pid

while [ -z "$DRWEBD_ADDRESS" ]; do
    echo
    echo -n "Enter the drwebd address [$DRWEBD_ADDRESS_DEFAULT]: "
    read DRWEBD_ADDRESS
    [ -n "$DRWEBD_ADDRESS" ] || DRWEBD_ADDRESS=$DRWEBD_ADDRESS_DEFAULT
    echo "$DRWEBD_ADDRESS" | egrep '^(inet:[0-9]+@|local:|pid:)[^\s]+' >/dev/null
    if [ "$?" != "0" ]; then
        echo "The address is invalid."
        DRWEBD_ADDRESS=
    fi
done

# Escape '@' to prevent Perl substitution breakage.
DRWEBD_ADDRESS=`echo $DRWEBD_ADDRESS | sed 's/@/\\\\@/'`

echo
echo "Setting 'Address' to '$DRWEBD_ADDRESS' in [DaemonCommunication] of $NSS_CONF ."
if [ -z "$drwebd_address" ]; then
    IniSectProc $NSS_CONF_NEW DaemonCommunication "if (!\$added) { \$_=\"# Added by Dr.Web's $SCRIPT_NAME\nAddress = $DRWEBD_ADDRESS\n\n\".\$_; \$added=1 }"
else
    IniSectProc $NSS_CONF_NEW DaemonCommunication "s#^(\\s*Address\\s*=\\s*).*#\$1$DRWEBD_ADDRESS#;"
fi

AskUpdateConfig $NSS_CONF $NSS_CONF_NEW dont_ask

CRON_DRWEB_UPDATE=/etc/cron.d/drweb-update
echo
echo "Checking if cron user in $CRON_DRWEB_UPDATE is root."
if [ -f "$CRON_DRWEB_UPDATE" ]; then
    cron_usr=`awk '{print $6}' $CRON_DRWEB_UPDATE`
    echo $cron_usr | grep -v ^root$ >/dev/null 2>&1
    if [ "$?" = "0" ]; then
        echo "Changing cron user in $CRON_DRWEB_UPDATE from $cron_usr to root."

        CRON_DRWEB_UPDATE_NEW=`MakeTmpFile`
        awk '{print $1" "$2" "$3" "$4" "$5" root "$7}' $CRON_DRWEB_UPDATE > \
            $CRON_DRWEB_UPDATE_NEW

        # Not using AskUpdateConfig here because we don't want backup in cron.d.
        mv $CRON_DRWEB_UPDATE_NEW $CRON_DRWEB_UPDATE
    fi
else
    echo "No such file."
fi

echo
echo "Changing drwebd process ownership."
sed 's|\(drwebd.*\)drweb:drweb\(.*\)$|\1root:drweb\2|' $DAEMON_MMC > \
    $DAEMON_MMC_NEW
AskUpdateConfig $DAEMON_MMC $DAEMON_MMC_NEW dont_ask

echo
echo "Configuration of drweb-nss is completed successfully."
echo
exit 0
